"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class HTTPHeaders {
    constructor(header, value, info = "N/A") {
        this.header = header;
        this.value = value;
        this.info = info;
    }
}
exports.HTTPHeaders = HTTPHeaders;
class SecurityHeaders {
    static secureHeaders(options) {
        let headers = [];
        options = parseOptions(options);
        if (options.server) {
            if (typeof options.server == "string") {
                SecurityHeaders.server.value = options.server;
            }
            headers.push(SecurityHeaders.server);
        }
        if (options.hsts) {
            if (typeof options.hsts == "string") {
                SecurityHeaders.httpStrictTransportSecurity.value = options.hsts;
            }
            headers.push(SecurityHeaders.httpStrictTransportSecurity);
        }
        if (options.xfo) {
            if (typeof options.xfo == "string") {
                SecurityHeaders.xFrameOptions.value = options.xfo;
            }
            headers.push(SecurityHeaders.xFrameOptions);
        }
        if (options.xxp) {
            if (typeof options.xxp == "string") {
                SecurityHeaders.xXssProtection.value = options.xxp;
            }
            headers.push(SecurityHeaders.xXssProtection);
        }
        if (options.content) {
            if (typeof options.content == "string") {
                SecurityHeaders.xContentTypeOptions.value = options.content;
            }
            headers.push(SecurityHeaders.xContentTypeOptions);
        }
        if (options.csp) {
            if (typeof options.csp == "string") {
                SecurityHeaders.contentSecurityPolicy.value = options.csp;
            }
            headers.push(SecurityHeaders.contentSecurityPolicy);
        }
        if (options.referrer) {
            if (typeof options.referrer == "string") {
                SecurityHeaders.referrerPolicy.value = options.referrer;
            }
            headers.push(SecurityHeaders.referrerPolicy);
        }
        if (options.cache) {
            if (typeof options.cache == "string") {
                SecurityHeaders.cacheControl.value = options.cache;
            }
            else {
                headers.push(SecurityHeaders.pragma);
                headers.push(SecurityHeaders.expires);
            }
            headers.push(SecurityHeaders.cacheControl);
        }
        if (options.feature) {
            if (typeof options.feature == "string") {
                SecurityHeaders.featurePolicy.value = options.feature;
            }
            headers.push(SecurityHeaders.featurePolicy);
        }
        return headers;
    }
}
SecurityHeaders.server = new HTTPHeaders("Server", "NULL");
SecurityHeaders.httpStrictTransportSecurity = new HTTPHeaders("Strict-Transport-Security", "max-age=63072000; includeSubdomains", "Ensure application communication is sent over HTTPS");
SecurityHeaders.xFrameOptions = new HTTPHeaders("X-Frame-Options", "SAMEORIGIN", "Disable framing from different origins (clickjacking defense)");
SecurityHeaders.xXssProtection = new HTTPHeaders("X-XSS-Protection", "1; mode=block", "Enable Cross-Site Scripting filters");
SecurityHeaders.xContentTypeOptions = new HTTPHeaders("X-Content-Type-Options", "nosniff", "Prevent MIME-sniffing");
SecurityHeaders.contentSecurityPolicy = new HTTPHeaders("Content-Security-Policy", "script-src 'self'; object-src 'self'", "Prevent Cross-site injections");
SecurityHeaders.referrerPolicy = new HTTPHeaders("Referrer-Policy", "no-referrer, strict-origin-when-cross-origin", "Enable full referrer if same origin, remove path for cross origin and disable referrer in unsupported browsers");
SecurityHeaders.cacheControl = new HTTPHeaders("Cache-control", "no-cache, no-store, must-revalidate, max-age=0", "Prevent cacheable HTTPS response");
SecurityHeaders.pragma = new HTTPHeaders("Pragma", "no-cache", "Prevent cacheable HTTPS response");
SecurityHeaders.expires = new HTTPHeaders("Expires", "0", "Prevent cacheable HTTPS response");
SecurityHeaders.featurePolicy = new HTTPHeaders("Feature-Policy", "accelerometer 'none'; ambient-light-sensor 'none'; autoplay 'none'; camera 'none'; encrypted-media 'none'; fullscreen 'none'; geolocation 'none'; gyroscope 'none'; magnetometer 'none'; microphone 'none'; midi 'none'; payment 'none'; picture-in-picture 'none'; speaker 'none'; sync-xhr 'none'; usb 'none'; vr 'none'", "Disable browser features and APIs");
exports.SecurityHeaders = SecurityHeaders;
function parseOptions(options) {
    let defaultOptions = {
        server: false,
        hsts: true,
        xfo: true,
        xxp: true,
        content: true,
        csp: false,
        referrer: true,
        cache: true,
        feature: false
    };
    for (let property in options) {
        if (options.hasOwnProperty(property)) {
            defaultOptions[property] = options[property];
        }
    }
    return defaultOptions;
}
exports.parseOptions = parseOptions;
function headerObject(options) {
    const headers = SecurityHeaders.secureHeaders(options);
    let secureHeaders = {};
    for (let header of headers) {
        secureHeaders[header.header] = header.value;
    }
    return secureHeaders;
}
exports.headerObject = headerObject;
function setHeaderTuple(response, options) {
    const headers = SecurityHeaders.secureHeaders(options);
    for (let header of headers) {
        response.header(header.header, header.value);
    }
}
exports.setHeaderTuple = setHeaderTuple;
